package dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Hashtable;

import model.DataFile;
import exception.PersistenceException;
import factory.JDBCDAOFactory;

public class JDBCDataFileDAO implements DataFileDAO {

	private Statement stm;
	private PreparedStatement stmt;
	private ResultSet rs; 
	private Connection conn;
	
	public enum Sql {
		RECUPERAR_TODOS("SELECT * FROM DATAFILE"),
		
		RECUPERAR_POR_ID("SELECT * FROM DATAFILE WHERE ID_DATAFILE = ?"),
		
		RECUPERAR_POR_PATH("SELECT * FROM DATAFILE WHERE PATH_DATAFILE = ?"),
		
		SALVAR("INSERT INTO DATAFILE (PATH_DATAFILE, TYPE_DATAFILE) VALUES(?,?)"),
		
		REMOVER("DELETE FROM DATAFILE WHERE ID_DATAFILE = ?"),
		
		ATUALIZAR("UPDATE DATAFILE SET PATH_DATAFILE = ?, TYPE_DATAFILE = ? " +
				"WHERE ID_DATAFILE = ?");
		
		Sql(String sql) {
			this.sql = sql;
		}
		
		public String toString() {
			return sql;
		}
		
		private String sql;
	}
	
	public JDBCDataFileDAO() {
		this.openPersistence();
	}
	
	/**
	 * Abre o mecanismo de persistncia
	 */
	@SuppressWarnings("unchecked")
	public <T> T openPersistence() {
		conn = ((Connection) JDBCDAOFactory.getInstance().openConnection());
		if (stm == null)
			try {
				stm = conn.createStatement();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		return (T) stm;
	}

	/**
	 * Fecha o mecanismo de persistncia
	 */
	public <T> void closePersistence(T persistence) {
		if (persistence instanceof ResultSet) {
			rs = (ResultSet) persistence;
			try {
				if (rs != null)
					rs.close();
				
				if (stmt != null) 
					stmt.close();
				
				if (stm != null) 
					stm.close();
				
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Atualiza o DataFile no mecanismo de persistencia
	 * @param	datafile	O datafile a ser atualizado	
	 */
	public void atualizar(DataFile datafile) throws PersistenceException {
		try {
			stmt = conn.prepareStatement(Sql.ATUALIZAR.toString());
			stmt.setString(1, datafile.getPath());
			stmt.setString(2, datafile.getType());
			stmt.setInt(3, datafile.getId());
			stmt.execute();
			//conn.commit();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Recupera o DataFile de id idDatafile
	 * @param	idDatafile		O id do DataFile
	 * @return	O DataFile de id idDataFile
	 * 			null, caso no exista
	 */
	public DataFile recuperarPorId(Integer idDataFile)
			throws PersistenceException {
		DataFile df = null;		
		try {
			stmt = conn.prepareStatement(Sql.RECUPERAR_POR_ID.toString());
			stmt.setInt(1, idDataFile);
			rs = stmt.executeQuery();
			while(rs.next()) {
				df = new DataFile();
				String path = rs.getString("PATH_DATAFILE");
				String type = rs.getString("TYPE_DATAFILE");

				df.setPath(path);
				df.setType(type);
				df.setId(idDataFile);
				
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return df;
	}
	
	/**
	 * Recupera o DataFile de caminho pathDataFile
	 * @param	pathDataFile	O caminho do DataFile
	 * @return	O DataFile de caminho pathDataFile
	 * 			null, caso no exista
	 */
	public DataFile recuperarPorPath(String pathDataFile) throws PersistenceException {
		DataFile df = null;		
		try {
			stmt = conn.prepareStatement(Sql.RECUPERAR_POR_PATH.toString());
			stmt.setString(1, pathDataFile);
			rs = stmt.executeQuery();
			while(rs.next()) {
				df = new DataFile();
				//String path = rs.getString("PATH_DATAFILE");
				String type = rs.getString("TYPE_DATAFILE");
				Integer id = rs.getInt("ID_DATAFILE");
				df.setPath(pathDataFile);
				df.setType(type);
				df.setId(id);
				
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return df;
	}

	/**
	 * Remove o DataFile de id idDataFile
	 * @param	idDataFile		O id do DataFile
	 */
	public void remover(Integer idDataFile)
			throws PersistenceException {
		DataFile df = null;		
		try {
			df = this.recuperarPorId(idDataFile);			
			stmt = conn.prepareStatement(Sql.REMOVER.toString());
			stmt.setInt(1, idDataFile);
			if (df != null) {
				stmt.executeUpdate();
				//conn.commit();
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Salva o datafile
	 * @param	datafile		O DataFile a ser salvo
	 * @return	O id do DataFile salvo
	 */
	public Integer salvar(DataFile datafile) throws PersistenceException {
		try {
			stmt = conn.prepareStatement(Sql.SALVAR.toString());
			stmt.setString(1, datafile.getPath());
			stmt.setString(2, datafile.getType());
			stmt.executeUpdate();
			//conn.commit();
			ResultSet generateKeys = stmt.getGeneratedKeys();
			if (generateKeys.next()) {
				int id = generateKeys.getInt(1);
				datafile.setId(id);
			}
			return datafile.getId();			
		} catch (SQLException e) {
			System.out.println(e.getMessage());
			return null;			
		}
	}

	/**
	 * Recupera todos os DataFile
	 * @return	Um Hashtable com todos os datafiles
	 */
	public Hashtable<String, DataFile> recuperarTodos() throws PersistenceException {
		try {
			rs = stm.executeQuery(Sql.RECUPERAR_TODOS.toString());
		} catch (SQLException e) {
			e.printStackTrace();
		}
		Hashtable<String, DataFile> datafiles = new Hashtable<String, DataFile>();
		try {
			while(rs.next()) {
				Integer idDataFile = rs.getInt("ID_DATAFILE");
				String path = rs.getString("PATH_DATAFILE");
				String type = rs.getString("TYPE_DATAFILE");

				DataFile df = new DataFile();
				
				df.setId(idDataFile);
				df.setPath(path);
				df.setType(type);

				
				//datafiles.add(df);
				datafiles.put(df.getId().toString(), df);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return datafiles;
	}

}
