package dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Hashtable;

import model.Tag;
import exception.PersistenceException;
import factory.JDBCDAOFactory;

public class JDBCTagDAO implements TagDAO {

	private Statement stm;
	private PreparedStatement stmt;
	private ResultSet rs; 
	private Connection conn;
	
	public enum Sql {
		RECUPERAR_TODOS("SELECT * FROM TAG"),
		
		RECUPERAR_POR_ID("SELECT * FROM TAG WHERE ID_TAG = ?"),
		
		RECUPERAR_POR_NOME("SELECT * FROM TAG WHERE NAME_TAG = ?"),
		
		SALVAR("INSERT INTO TAG (NAME_TAG) VALUES(?)"),
		
		REMOVER("DELETE FROM TAG WHERE ID_TAG = ?"),
		
		ATUALIZAR("UPDATE TAG SET NAME_TAG = ? WHERE ID_TAG = ?");
		
		Sql(String sql) {
			this.sql = sql;
		}
		
		public String toString() {
			return sql;
		}
		
		private String sql;
	}
	
	public JDBCTagDAO() {
		this.openPersistence();
	}
	
	/**
	 * Abre a persistencia
	 */
	@SuppressWarnings("unchecked")
	public <T> T openPersistence() {
		conn = ((Connection) JDBCDAOFactory.getInstance().openConnection());
		if (stm == null)
			try {
				stm = conn.createStatement();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		return (T) stm;
	}

	/**
	 * Fecha a persistencia
	 */
	public <T> void closePersistence(T persistence) {
		if (persistence instanceof ResultSet) {
			rs = (ResultSet) persistence;
			try {
				if (rs != null)
					rs.close();
				
				if (stmt != null) 
					stmt.close();
				
				if (stm != null) 
					stm.close();
				
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	/**
	 * Atualiza a Tag no mecanismo de persistencia
	 * @param	tag		A Tag a ser atualizado	
	 */
	 
	public void atualizar(Tag tag) throws PersistenceException {
		try {
			stmt = conn.prepareStatement(Sql.ATUALIZAR.toString());
			stmt.setString(1, tag.getName());
			stmt.setInt(2, tag.getId());
			stmt.execute();
			//conn.commit();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	/**
	 * Recupera a Tag de nome nomeTag
	 * @param	nomeTag		O nome da Tag
	 * @return	A Tag de nome nomeTag
	 * 			null, caso no exista
	 */
	public Tag recuperarPorNome(String nomeTag) throws PersistenceException {
		Tag tag = null;		
		try {
			stmt = conn.prepareStatement(Sql.RECUPERAR_POR_NOME.toString());
			stmt.setString(1, nomeTag);
			rs = stmt.executeQuery();
			while(rs.next()) {
				tag = new Tag();
				Integer id = rs.getInt("ID_TAG");

				tag.setId(id);
				tag.setName(nomeTag);
				
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return tag;
	}
	
	/**
	 * Recupera a Tag de id idTag
	 * @param	idTag		O id da Tag
	 * @return	A Tag de id idTag
	 * 			null, caso no exista
	 */
	public Tag recuperarPorId(Integer idTag) throws PersistenceException {
		Tag tag = null;		
		try {
			stmt = conn.prepareStatement(Sql.RECUPERAR_POR_ID.toString());
			stmt.setInt(1, idTag);
			rs = stmt.executeQuery();
			while(rs.next()) {
				tag = new Tag();
				String name = rs.getString("NAME_TAG");
		
				tag.setName(name);
				tag.setId(idTag);
				
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return tag;
	}



	/**
	 * Remove a Tag de id codigoTag
	 * @param	idTag		O id do DataFile
	 */
	public void remover(Integer idTag)
			throws PersistenceException {
		Tag t = null;		
		try {
			t = this.recuperarPorId(idTag);			
			stmt = conn.prepareStatement(Sql.REMOVER.toString());
			stmt.setInt(1, idTag);
			if (t != null) {
				stmt.executeUpdate();
				//conn.commit();
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Salva a Tag
	 * @param	tag		Tag a ser salva
	 * @return	O id da Tag salva
	 */
	public Integer salvar(Tag tag) throws PersistenceException {
		try {
			stmt = conn.prepareStatement(Sql.SALVAR.toString());
			stmt.setString(1, tag.getName());
			stmt.executeUpdate();
			//conn.commit();
			ResultSet generateKeys = stmt.getGeneratedKeys();
			if (generateKeys.next()) {
				int id = generateKeys.getInt(1);
				tag.setId(id);
			}
			return tag.getId();			
		} catch (SQLException e) {
			System.out.println(e.getMessage());
			return null;			
		}
	}

	/**
	 * Recupera todos as Tags
	 * @return	Um Hashtable com todos as tags
	 */
	public Hashtable<String, Tag> recuperarTodos() throws PersistenceException {
		try {
			rs = stm.executeQuery(Sql.RECUPERAR_TODOS.toString());
		} catch (SQLException e) {
			e.printStackTrace();
		}
		Hashtable<String, Tag> tags = new Hashtable<String, Tag>();
		try {
			while(rs.next()) {
				Integer idTag = rs.getInt("ID_TAG");
				String name = rs.getString("NAME_TAG");
				Tag t = new Tag();
				
				t.setId(idTag);
				t.setName(name);
			
				//tags.add(t);
				tags.put(t.getName(), t);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return tags;
	}

}
